import androidx.compose.animation.ExperimentalSharedTransitionApi
import androidx.compose.animation.SharedTransitionLayout
import androidx.compose.foundation.isSystemInDarkTheme
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.lazy.grid.GridCells
import androidx.compose.foundation.lazy.grid.LazyVerticalGrid
import androidx.compose.foundation.lazy.grid.rememberLazyGridState
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.MoreVert
import androidx.compose.material3.Button
import androidx.compose.material3.DropdownMenu
import androidx.compose.material3.DropdownMenuItem
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateListOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.rememberNavController
import details.RecipeDetails
import model.TagType
import recipeslist.RecipesListScreen
import sensor.SensorManager
import ui.theme.MainTheme
import view.BookShelf
import view.FilterCard
import view.HomeScreen
import view.InputFieldState
import view.getFilteredRecipeList
import view.getRecipeList


enum class Theme {
    Auto,
    Light,
    Dark
}

enum class RecipeAppScreen {
    List, Details,
}

@OptIn(ExperimentalSharedTransitionApi::class)
@Composable
fun App(
    sensorManager: SensorManager?,
    onClose: () -> Unit,
    isLarge: Boolean = false,
    isWebApp: Boolean = false) {

    val navController = rememberNavController()
    var theme by remember { mutableStateOf(Theme.Auto) }

    val onThemeToggle = {
        theme = when (theme) {
            Theme.Auto -> Theme.Light
            Theme.Light -> Theme.Dark
            Theme.Dark -> Theme.Auto
        }
    }
    val isDarkTheme: Boolean? = when (theme) {
        Theme.Dark -> true
        Theme.Light -> false
        Theme.Auto -> null
    }

    MainTheme(isDarkTheme ?: isSystemInDarkTheme()) {

        var show by remember { mutableStateOf(false) }
        var searchBar by remember { mutableStateOf(false) }
        var search by remember { mutableStateOf("") }
        val tags = remember { mutableStateListOf<String>() }
        val recipeTags by remember { mutableStateOf(mutableMapOf<String, TagType>()) }
        var book by remember { mutableStateOf("") }
        var returnAnyMatch by remember { mutableStateOf(false) }

        for (recipe in getRecipeList()) {
            for (tag in recipe.tags) {
                if (!recipeTags.contains(tag.key))
                    recipeTags[tag.key] = tag.value
            }
        }

        if (show) {
            FilterCard(
                recipeTags = recipeTags.keys.toList(),
                activeTags = tags,
            ) {
                for(item in it)
                    if(tags.contains(item).not())
                        tags.add(item)

                show = false
            }
        }

        val filteredItems = getFilteredRecipeList(tags, search, book, returnAnyMatch)
        val recipeCount = getRecipeList().size
        var currentRecipe = getRecipeList().first()

        if (filteredItems.isNotEmpty())
            currentRecipe = filteredItems.first()

        Scaffold {
            Column {
                if (searchBar) {
                        Row {
                            Button(
                                onClick = {
                                    show = true
                                },
                            ) {
                                Text(text = "Filter")
                            }
                            InputFieldState(
                                value = search,
                                label = "Search",
                                modifier = Modifier
                                    .width(if (isLarge) 400.dp else 200.dp)
                            ) {
                                search = it
                            }

                            Spacer(modifier = Modifier.weight(1f))

                            MainDropDown(
                                isLarge,
                                returnAnyMatch,
                                theme,
                                onClose = onClose,
                                onBack = {navController.navigateUp()},
                                onTag = {returnAnyMatch = !returnAnyMatch},
                                onTheme = { onThemeToggle.invoke() },
                                onRandom = {
                                    currentRecipe = filteredItems.random()
                                    navController.navigate(RecipeAppScreen.Details.name)
                                },
                            )
                        }
                        val listState = rememberLazyGridState()
                        LazyVerticalGrid(
                            state = listState, columns = GridCells.Adaptive(minSize = 110.dp)
                        )
                        {
                            items(tags.size) {
                                Button(onClick = { tags.remove(tags[it]) }) {
                                    Text(text = tags[it])
                                }
                            }
                        }
                    if(isLarge && (tags.isNotEmpty() || search.isNotEmpty() || book.isNotEmpty()))
                        Text("Showing ".plus(filteredItems.size).plus( " out of ").plus(recipeCount).plus(" recipes"))
                    }

                SharedTransitionLayout {
                    NavHost(
                        navController = navController,
                        startDestination = "Home",
                        modifier = Modifier.fillMaxSize()
                    ) {
                        composable(route = "Home")
                        {
                            searchBar = false
                            HomeScreen(
                                isLarge = isLarge,
                                onGo = {
                                    if(isWebApp)
                                        navController.navigate(RecipeAppScreen.List.name)
                                    else
                                        navController.navigate("BookShelf")
                                },
                                onSurprise = {
                                    currentRecipe = filteredItems.random()
                                    navController.navigate(RecipeAppScreen.Details.name)
                                },
                                isWebApp = isWebApp
                            )
                        }
                        composable(route = "BookShelf")
                        {
                            searchBar = false
                            BookShelf(
                                isLarge = isLarge,
                                tags = recipeTags,
                                onClick = { lockedTag ->
                                    book = lockedTag
                                    if(book == "SURPRISE") {
                                        currentRecipe = filteredItems.random()
                                        navController.navigate(RecipeAppScreen.Details.name)
                                    }else{
                                    navController.navigate(RecipeAppScreen.List.name) }

                                }
                            )
                        }
                        composable(
                            route = RecipeAppScreen.List.name
                        ) {
                            if(isWebApp.not())
                                searchBar = true

                            RecipesListScreen(
                                isLarge = isLarge,
                                items = filteredItems,
                                onClick = { recipe ->
                                    currentRecipe = recipe
                                    navController.navigate(RecipeAppScreen.Details.name)
                                })
                        }
                        composable(route = RecipeAppScreen.Details.name) {
                            searchBar = false
                            RecipeDetails(
                                isLarge = isLarge,
                                isWebApp = isWebApp,
                                sensorManager = sensorManager,
                                recipe = currentRecipe,
                                goBack = {
                                    navController.popBackStack()
                                })
                        }
                    }
                }
            }
        }
    }
}

@Composable
fun MainDropDown(
    isLarge: Boolean,
    andOr: Boolean,
    theme: Theme,
    onTag: () -> Unit,
    onTheme: () -> Unit,
    onRandom: () -> Unit,
    onBack: () -> Unit,
    onClose: () -> Unit,
) {
    var expanded by remember { mutableStateOf(false) }

    Box(
        modifier = Modifier
            .padding(16.dp)
    ) {
        IconButton(onClick = { expanded = !expanded }) {
            Icon(imageVector = Icons.Default.MoreVert, contentDescription = "More Options")
        }
        DropdownMenu(
            expanded = expanded,
            onDismissRequest = { expanded = false }
        ) {
            DropdownMenuItem(
                text = { Text(theme.name) },
                onClick = onTheme
            )
            DropdownMenuItem(
                text = {Text(if(andOr) "OR" else "AND")},
                onClick = onTag
            )
            DropdownMenuItem(
                text = { Text("Random Filtered Recipe") },
                onClick = onRandom
            )
            if (isLarge) {

                DropdownMenuItem(
                    text = {Text("Go Back")},
                    onClick = onBack
                )

                DropdownMenuItem(
                    text = { Text("Close app") },
                    onClick = onClose
                )
            }
        }
    }
}
